package nl.iobyte.themepark.api;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.attraction.AttractionService;
import nl.iobyte.themepark.api.config.ConfigurationManager;
import nl.iobyte.themepark.api.database.DatabaseService;
import nl.iobyte.themepark.api.event.EventDispatcher;
import nl.iobyte.themepark.api.load.DataLoadService;
import nl.iobyte.themepark.api.menu.MenuService;
import nl.iobyte.themepark.api.ridecount.RideCountService;
import nl.iobyte.themepark.api.sign.SignManager;
import nl.iobyte.themepark.api.sync.SyncService;
import nl.iobyte.themepark.api.sync.enums.SyncType;

public class ThemeParkAPI {

    //Data
    private ConfigurationManager configurationManager;
    private final DataLoadService dataLoadService = new DataLoadService();
    private final SyncService syncService = new SyncService();
    private final DatabaseService databaseService = new DatabaseService();
    private final MenuService menuService = new MenuService();
    private EventDispatcher eventDispatcher;

    //Attraction
    private final AttractionService attractionService = new AttractionService();
    private final RideCountService rideCountService = new RideCountService();
    private final SignManager signManager = new SignManager();

    /**
     * Enable the API
     */
    public void enable() {
        //Prepare Configuration
        configurationManager = new ConfigurationManager(ThemePark.getInstance());

        //Load data
        eventDispatcher = new EventDispatcher(ThemePark.getInstance());
        dataLoadService.init();
        syncService.init();
    }

    /**
     * Disable the API
     */
    public void disable() {
        databaseService.stop();
        syncService.callSync(SyncType.HALT);
    }

    /**
     * Get the configuration manager
     * @return ConfigurationManager
     */
    public ConfigurationManager getConfigurationManager() {
        return configurationManager;
    }

    /**
     * Get the attraction service
     * @return AttractionService
     */
    public AttractionService getAttractionService() {
        return attractionService;
    }

    /**
     * Get the data load service
     * @return DataLoadService
     */
    public DataLoadService getDataLoadService() {
        return dataLoadService;
    }

    /**
     * Get the menu service
     * @return MenuService
     */
    public MenuService getMenuService() {
        return menuService;
    }

    /**
     * Get the ride count service
     * @return RideCountService
     */
    public RideCountService getRideCountService() {
        return rideCountService;
    }

    /**
     * Get the sign manager
     * @return SignManager
     */
    public SignManager getSignManager() {
        return signManager;
    }

    /**
     * Get the database service
     * @return DatabaseService
     */
    public DatabaseService getDatabaseService() {
        return databaseService;
    }

    /**
     * Get the event dispatcher
     * @return EventDispatcher
     */
    public EventDispatcher getEventDispatcher() {
        return eventDispatcher;
    }

}
