package nl.iobyte.themepark.api.attraction.enums;

import com.cryptomorin.xseries.XMaterial;
import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.event.status.*;
import org.bukkit.inventory.ItemStack;

public enum Status {

    GLOBAL(0, "&1", "#3498db", "Global", "STAINED_CLAY:3", false),
    CONSTRUCTION(1, "&7", "#95a5a6", "Under Contruction", "STAINED_CLAY:1",false),
    OPEN(2, "&a", "#4cd137", "Open", "STAINED_CLAY:5", true),
    CLOSED(3, "&4", "#e84118", "Closed", "STAINED_CLAY:14", false),
    MAINTENANCE(4, "&6", "#f39c12", "Under Maintenance", "STAINED_CLAY:4", false),
    MALFUNCTION(5, "&5", "#8e44ad", "Malfunction", "STAINED_CLAY:11", false);

    private final int id;
    private String color, hex_color, name;
    private XMaterial material;
    private boolean teleport;
    Status(int id, String color, String hex_color, String name, String material, boolean teleport) {
        this.id = id;
        this.color = color;
        this.hex_color = hex_color;
        this.name = name;
        this.material = XMaterial.matchXMaterial(material).orElse(XMaterial.RED_CONCRETE);
        this.teleport = teleport;
    }

    /**
     * Get ID of Status
     * @return Integer
     */
    public int getID() {
        return id;
    }

    /**
     * Get Color of Status
     * @return String
     */
    public String getColor() {
        return color;
    }

    /**
     * Set Color of Status
     * @param color String
     */
    public void setColor(String color) {
        String old = this.color;
        this.color = color;
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new StatusColorChangeEvent(
                this,
                old,
                color
        ));
    }

    /**
     * Get HexColor of Status
     * @return String
     */
    public String getHexColor() {
        return hex_color;
    }

    /**
     * Set HexColor of Status
     * @param hex_color String
     */
    public void setHexColor(String hex_color) {
        String old = this.hex_color;
        this.hex_color = hex_color;
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new StatusHexColorChangeEvent(
                this,
                old,
                hex_color
        ));
    }

    /**
     * Get Name of Status
     * @return String
     */
    public String getName() {
        return name;
    }

    /**
     * Set Name of Status
     * @param name String
     */
    public void setName(String name) {
        String old = this.name;
        this.name = name;
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new StatusNameChangeEvent(
                this,
                old,
                name
        ));
    }

    /**
     * Get XMaterial of Status
     * @return XMaterial
     */
    public XMaterial getMaterial() {
        return material;
    }

    /**
     * Set Material of Status
     * @param material Material
     */
    public void setMaterial(XMaterial material) {
        XMaterial old = this.material;
        this.material = material;
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new StatusMaterialChangeEvent(
                this,
                old,
                material
        ));
    }

    /**
     * Get Item of Status
     * @return ItemStack
     */
    public ItemStack getItem() {
        return material.parseItem();
    }

    /**
     * Can Teleport with Status
     * @return Boolean
     */
    public boolean canTeleport() {
        return teleport;
    }

    /**
     * Set if can Teleport with Status
     * @param teleport Boolean
     */
    public void setCanTeleport(boolean teleport) {
        boolean old = this.teleport;
        this.teleport = teleport;
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new StatusTeleportChangeEvent(
                this,
                old,
                teleport
        ));
    }

    /**
     * Get Status by name of enum
     * @param str String
     * @return Status
     */
    public static Status get(String str) {
        if(str == null || str.isEmpty())
            return null;

        try {
            return Status.valueOf(str.toUpperCase());
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Get Status by ID
     * @param id Integer
     * @return String
     */
    public static Status get(int id) {
        if(id <= 1 || id > values().length)
            return null;

        for(Status status : values())
            if(status.id == id)
                return status;

        return null;
    }

}
