package nl.iobyte.themepark.api.attraction.objects;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.attraction.enums.Status;
import nl.iobyte.themepark.api.event.attraction.*;
import org.bukkit.Location;

public class Attraction {

    private final String id;
    private String region_id, name,cover;
    private Status status;
    private transient Location location;

    public Attraction(String id, String region_id, String name, String cover, Status status, Location location) {
        this.id = id;
        this.region_id = region_id;
        this.name = name;
        this.cover = cover;
        this.status = status;
        this.location = location;
    }

    /**
     * Get ID of Attraction
     * @return String
     */
    public String getID() {
        return id;
    }

    /**
     * Get RegionID of Attraction
     * @return String
     */
    public String getRegionID() {
        return region_id;
    }

    /**
     * Set RegionID of Attraction
     * @param region_id String
     */
    public void setRegionID(String region_id) {
        String old = this.region_id;
        this.region_id = region_id;
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new AttractionRegionChangeEvent(
                this,
                old,
                region_id
        ));
    }

    /**
     * Get Name of Attraction
     * @return String
     */
    public String getName() {
        return name;
    }

    /**
     * Set Name of Attraction
     * @param name String
     */
    public void setName(String name) {
        String old = this.name;
        this.name = name;
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new AttractionNameChangeEvent(
                this,
                old,
                name
        ));
    }

    /**
     * Get Cover of Attraction
     * @return String
     */
    public String getCover() {
        return cover;
    }

    /**
     * Set Cover of Attraction
     * @param cover String
     */
    public void setCover(String cover) {
        String old = this.cover;
        this.cover = cover;
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new AttractionCoverChangeEvent(
                this,
                old,
                cover
        ));
    }

    /**
     * Get Status of Attraction
     * @return Status
     */
    public Status getStatus() {
        return status;
    }

    /**
     * Set Status of Attraction
     * @param status Attraction
     */
    public void setStatus(Status status) {
        Status old = this.status;
        this.status = status;
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new AttractionStatusChangeEvent(
                this,
                old,
                status
        ));
    }

    /**
     * Get Location of Attraction
     * @return Location
     */
    public Location getLocation() {
        return location;
    }

    /**
     * Set Location of Attraction
     * @param location Attraction
     */
    public void setLocation(Location location) {
        Location old = this.location;
        this.location = location;
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new AttractionLocationChangeEvent(
                this,
                old,
                location
        ));
    }

}
