package nl.iobyte.themepark.api.load.objects;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.ThemeParkAPI;
import nl.iobyte.themepark.api.config.ConfigurationManager;
import nl.iobyte.themepark.api.config.enums.StorageKey;
import nl.iobyte.themepark.api.database.DatabaseService;
import nl.iobyte.themepark.api.database.objects.types.MySQLDatabase;
import nl.iobyte.themepark.api.database.objects.types.SQLiteDatabase;
import nl.iobyte.themepark.api.load.interfaces.IDataLoader;

public class DatabaseLoader implements IDataLoader {

    public void load() {
        ThemeParkAPI api = ThemePark.getInstance().getAPI();
        DatabaseService service = api.getDatabaseService();
        ConfigurationManager manager = api.getConfigurationManager();

        //Add SQLite Database
        service.addDatabase(
                "local",
                new SQLiteDatabase("sqlite")
        );

        if(manager.getBoolean(StorageKey.MYSQL_ENABLED))
            service.addDatabase(
                    "remote",
                    new MySQLDatabase(
                            manager.getString(StorageKey.MYSQL_URL),
                            manager.getString(StorageKey.MYSQL_HOST),
                            manager.getInt(StorageKey.MYSQL_PORT),
                            manager.getString(StorageKey.MYSQL_NAME),
                            manager.getString(StorageKey.MYSQL_USERNAME),
                            manager.getString(StorageKey.MYSQL_PASSWORD)
                    )
            );

        //Create Database tables
        loadTables(service);
    }

    private void loadTables(DatabaseService service) {
        //Create SQLite Tables
        service.execute(
                "local",
                "CREATE TABLE IF NOT EXISTS counts(uuid VARCHAR(255) NOT NULL, attraction_id VARCHAR(255) NOT NULL, year SMALLINT(2), month SMALLINT(2), week SMALLINT(2), day SMALLINT(2), count SMALLINT(2), UNIQUE(uuid, attraction_id, year, day))",
                null
        );

        //Create MySQL Tables
        service.execute(
                "remote",
                "CREATE TABLE IF NOT EXISTS regions(id VARCHAR(255) NOT NULL, name VARCHAR(255) NOT NULL, PRIMARY KEY(id))",
                null
        );
        service.execute(
                "remote",
                "CREATE TABLE IF NOT EXISTS attractions(id VARCHAR(255) NOT NULL, region_id VARCHAR(255) NOT NULL, name VARCHAR(255) NOT NULL, cover VARCHAR(255) NOT NULL, status_id INT(11), PRIMARY KEY(id))",
                null
        );
        service.execute(
                "remote",
                "CREATE TABLE IF NOT EXISTS states(id INT(11), name VARCHAR(255) NOT NULL, color VARCHAR(7) NOT NULL, PRIMARY KEY(id))",
                null
        );
    }

}
