package nl.iobyte.themepark.api.sync;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.sync.enums.SyncType;
import nl.iobyte.themepark.api.sync.interfaces.ISync;
import nl.iobyte.themepark.api.sync.objects.ConfigurationSync;
import nl.iobyte.themepark.api.sync.objects.SignSync;
import org.bukkit.Bukkit;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

public class SyncService {

    private int TaskID = -1;
    private final Map<SyncType, ArrayList<ISync>> synchronisations = new HashMap<>();

    public SyncService() {
        addSynchronisation(new ConfigurationSync(), SyncType.POLL, SyncType.HALT);
        addSynchronisation(new SignSync(), SyncType.POLL, SyncType.HALT);
    }

    /**
     * Start Synchronisation Service
     */
    public void init() {
        if(TaskID != -1)
            return;

        //Boot
        callSync(SyncType.BOOT);

        //Register listeners for state changes
        long delay = 15 * 60 * 20; //Set to every 15 minutes
        TaskID = Bukkit.getScheduler().runTaskTimerAsynchronously(ThemePark.getInstance(), () -> callSync(SyncType.POLL), delay, delay).getTaskId();
    }

    /**
     * Add ISync to Synchronisation Service
     * @param sync ISync
     * @param types SyncType[]
     */
    public void addSynchronisation(ISync sync, SyncType... types) {
        if(sync == null || types == null || types.length == 0)
            return;

        for(SyncType type : types)
            if(type != null)
                Objects.requireNonNull(synchronisations.computeIfAbsent(
                        type,
                        syncType -> new ArrayList<>()
                )).add(sync);
    }

    public ArrayList<ISync> getOfType(SyncType type) {
        if(type == null)
            return null;

        return synchronisations.getOrDefault(type, new ArrayList<>());
    }

    public void callSync(SyncType type) {
        if(type == null)
            return;

        for(ISync sync : getOfType(type))
            sync.sync();
    }

}

