package nl.iobyte.themepark.commands;

import nl.iobyte.commandapi.CommandFactory;
import nl.iobyte.commandapi.interfaces.ICommandExecutor;
import nl.iobyte.commandapi.middlewares.PermissionMiddleware;
import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.config.enums.StorageKey;
import nl.iobyte.themepark.commands.executors.ConsoleExecutor;
import nl.iobyte.themepark.commands.executors.PlayerExecutor;
import nl.iobyte.themepark.commands.subcommands.HelpCommand;
import nl.iobyte.themepark.commands.subcommands.ItemCommand;
import nl.iobyte.themepark.commands.subcommands.MenuCommand;
import nl.iobyte.themepark.commands.subcommands.attraction.AttractionCommands;
import nl.iobyte.themepark.commands.subcommands.region.RegionCommands;
import nl.iobyte.themepark.commands.subcommands.ridecount.RideCountCommands;
import nl.iobyte.themepark.commands.subcommands.status.StatusCommands;
import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandMap;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import java.lang.reflect.Field;

public class ThemeParkCommand {

    //Load command data
    public ThemeParkCommand() {
        String cmd = ThemePark.getInstance().getAPI().getConfigurationManager().getString(StorageKey.CMD);

        CommandFactory factory = new CommandFactory(cmd);
        factory.addSubCommand(new HelpCommand(factory))
                .addSubCommand(new MenuCommand(factory.getName()));

        //Admin
        factory.addSubCommand(new ItemCommand(factory.getName()));

        //Region
        RegionCommands.load(factory);

        //Attraction
        AttractionCommands.load(factory);

        //Status
        StatusCommands.load(factory);

        //RideCount
        RideCountCommands.load(factory);

        //Middleware
        factory.addMiddleware(new PermissionMiddleware());

        //Register command
        try {
            Field bukkitCommandMap = Bukkit.getServer().getClass().getDeclaredField("commandMap");
            bukkitCommandMap.setAccessible(true);

            CommandMap commandMap = (CommandMap) bukkitCommandMap.get(Bukkit.getServer());
            commandMap.register(cmd, new Command(cmd) {
                public boolean execute(CommandSender sender, String label, String[] args) {
                    ICommandExecutor executor = new ConsoleExecutor(sender);
                    if(sender instanceof Player)
                        executor = new PlayerExecutor((Player) sender);

                    factory.onCommand(executor, args);
                    return true;
                }
            });
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

}
