package nl.iobyte.themepark.listeners;

import nl.iobyte.menuapi.item.ItemBuilder;
import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.ThemeParkAPI;
import nl.iobyte.themepark.api.config.ConfigurationManager;
import nl.iobyte.themepark.api.config.enums.StorageKey;
import nl.iobyte.themepark.api.message.Text;
import nl.iobyte.themepark.api.ridecount.enums.TotalType;
import nl.iobyte.themepark.scheduler.ThemeParkScheduler;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.event.inventory.InventoryType;
import org.bukkit.event.player.*;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import java.util.*;
import java.util.concurrent.CompletableFuture;

public class PlayerListener implements Listener {

    @EventHandler
    public void onJoin(PlayerJoinEvent e) {
        Player player = e.getPlayer();
        ThemeParkAPI api = ThemePark.getInstance().getAPI();

        //Handle Menu item onJoin
        doItem(player);

        //Load RideCount data
        TotalType type = api.getRideCountService().getType();
        String query;
        Map<Integer, Object> parameters;
        if(type == TotalType.TOTAL) {
            query = "SELECT count, attraction_id, (SELECT sum(count) FROM counts AS t1 WHERE t1.uuid=t2.uuid AND t1.attraction_id=t2.attraction_id GROUP BY t1.uuid) AS total_count FROM counts AS t2 WHERE uuid=?";
            parameters = new HashMap<>(){{
                put(1, player.getUniqueId().toString());
            }};
        } else {
            Calendar calendar = new GregorianCalendar();
            calendar.setTime(new Date());
            switch (type) {
                case DAILY:
                    query = "SELECT count, attraction_id, (SELECT sum(count) FROM counts AS t1 WHERE t1.uuid=t2.uuid AND t1.attraction_id=t2.attraction_id AND t1.year=t2.year AND t1.day=t2.day GROUP BY t1.uuid) AS total_count FROM counts AS t2 WHERE uuid=? AND year=? AND day=?";
                    parameters = new HashMap<>() {{
                        put(1, player.getUniqueId().toString());
                        put(2, calendar.get(Calendar.YEAR));
                        put(3, calendar.get(Calendar.DAY_OF_YEAR));
                    }};
                    break;
                case WEEKLY:
                    query = "SELECT count, attraction_id, (SELECT sum(count) FROM counts AS t1 WHERE t1.uuid=t2.uuid AND t1.attraction_id=t2.attraction_id AND t1.year=t2.year AND t1.week=t2.week GROUP BY t1.uuid) AS total_count FROM counts AS t2 WHERE uuid=? AND year=? AND week=?";
                    parameters = new HashMap<>() {{
                        put(1, player.getUniqueId().toString());
                        put(2, calendar.get(Calendar.YEAR));
                        put(3, calendar.get(Calendar.WEEK_OF_YEAR));
                    }};
                    break;
                case MONTHLY:
                    query = "SELECT count, attraction_id, (SELECT sum(count) FROM counts AS t1 WHERE t1.uuid=t2.uuid AND t1.attraction_id=t2.attraction_id AND t1.year=t2.year AND t1.month=t2.month GROUP BY t1.uuid) AS total_count FROM counts AS t2 WHERE uuid=? AND year=? AND month=?";
                    parameters = new HashMap<>() {{
                        put(1, player.getUniqueId().toString());
                        put(2, calendar.get(Calendar.YEAR));
                        put(3, calendar.get(Calendar.MONTH));
                    }};
                    break;
                case YEARLY:
                    query = "SELECT count, attraction_id, (SELECT sum(count) FROM counts AS t1 WHERE t1.uuid=t2.uuid AND t1.attraction_id=t2.attraction_id AND t1.year=t2.year GROUP BY t1.uuid) AS total_count FROM counts AS t2 WHERE uuid=? AND year=?";
                    parameters = new HashMap<>() {{
                        put(1, player.getUniqueId().toString());
                        put(2, calendar.get(Calendar.YEAR));
                    }};
                    break;
                default:
                    return;
            }
        }

        api.getDatabaseService().executeQueryAsync(
                "local",
                query,
                parameters
        ).setWhenSuccessful(result -> {
            if(result != null)
                for(Map<String, Object> row : result)
                    api.getRideCountService().setCount(
                            (String) row.get("attraction_id"),
                            player.getUniqueId(),
                            (int) row.get("count"),
                            (int) (row.get("total_count") == null ? row.get("count") : row.get("total_count"))
                    );
        });

    }

    @EventHandler
    public void onSwitch(PlayerChangedWorldEvent e) {
        //Handle Menu item onSwitch
        doItem(e.getPlayer());
    }

    @EventHandler
    public void onQuit(PlayerQuitEvent e) {
        Player player = e.getPlayer();
        UUID uuid = player.getUniqueId();

        //Clear Unwanted RideCount
        ThemeParkScheduler.runAsync(() -> ThemePark.getInstance().getAPI().getRideCountService().clearCount(uuid));
    }

    //Handle Menu item
    @EventHandler
    public void onInteract(PlayerInteractEvent e) {
        if(e.getItem() == null)
            return;

        ConfigurationManager manager = ThemePark.getInstance().getAPI().getConfigurationManager();
        Material material = Material.getMaterial(manager.getString(StorageKey.MENU_ITEM_MATERIAL));

        ItemStack item = e.getItem();
        if(item.getType() != material)
            return;

        if(!item.hasItemMeta())
            return;

        String name = Text.color(manager.getString(StorageKey.MENU_ITEM_NAME));
        if(!item.getItemMeta().getDisplayName().equals(name))
            return;

        if(e.getAction() == Action.LEFT_CLICK_AIR || e.getAction() == Action.LEFT_CLICK_BLOCK)
            return;

        e.setCancelled(true);
        Player player = e.getPlayer();
        Bukkit.dispatchCommand(player, manager.getString(StorageKey.CMD)+" menu");
    }

    //Handle Menu item drop
    @EventHandler
    public void onDrop(PlayerDropItemEvent e) {
        Player player = e.getPlayer();
        if(player.hasPermission("themepark.admin"))
            return;

        ConfigurationManager manager = ThemePark.getInstance().getAPI().getConfigurationManager();
        if(!manager.getBoolean(StorageKey.MENU_ITEM_ENABLED))
            return;

        Material material = Material.getMaterial(manager.getString(StorageKey.MENU_ITEM_MATERIAL));
        ItemStack item = e.getItemDrop().getItemStack();
        if(item.getType() != material)
            return;

        if(!item.hasItemMeta())
            return;

        String name = Text.color(manager.getString(StorageKey.MENU_ITEM_NAME));
        if(!item.getItemMeta().getDisplayName().equals(name))
            return;

        e.setCancelled(true);
    }

    @EventHandler
    public void onClick(InventoryClickEvent e) {
        Player player = (Player) e.getWhoClicked();
        if(player.hasPermission("themepark.admin"))
            return;

        Inventory inventory = e.getClickedInventory();
        if(inventory == null)
            return;
        
        if(inventory.getType() != InventoryType.PLAYER)
            return;

        ConfigurationManager manager = ThemePark.getInstance().getAPI().getConfigurationManager();
        if(!manager.getBoolean(StorageKey.MENU_ITEM_ENABLED))
            return;

        ItemStack item = e.getCursor();
        if(item == null)
            return;

        Material material = Material.getMaterial(manager.getString(StorageKey.MENU_ITEM_MATERIAL));
        if(item.getType() != material)
            return;

        if(!item.hasItemMeta())
            return;

        String name = Text.color(manager.getString(StorageKey.MENU_ITEM_NAME));
        if(!item.getItemMeta().getDisplayName().equals(name))
            return;

        e.setCancelled(true);
    }

    /**
     * Called on join / world switch
     * @param player Player
     */
    private void doItem(Player player) {
        ConfigurationManager manager = ThemePark.getInstance().getAPI().getConfigurationManager();

        CompletableFuture<ItemStack> fc = new CompletableFuture<>();
        Bukkit.getScheduler().runTaskAsynchronously(ThemePark.getInstance(), () -> {
            if(!manager.getBoolean(StorageKey.MENU_ITEM_ENABLED)) {
                fc.complete(null);
                return;
            }

            Material material = Material.getMaterial(manager.getString(StorageKey.MENU_ITEM_MATERIAL));
            short data = Short.parseShort(manager.getString(StorageKey.MENU_ITEM_DATA));
            String name = Text.color(manager.getString(StorageKey.MENU_ITEM_NAME));
            if(material == null || name.isEmpty()) {
                fc.complete(null);
                return;
            }

            ItemBuilder builder = new ItemBuilder(material, 1, data);
            builder.setName(name);
            if(manager.getBoolean(StorageKey.MENU_ITEM_CLEAR))
                player.getInventory().clear();

            List<String> worlds = manager.getStringList(StorageKey.MENU_ITEM_WORLDS);
            if(worlds != null && !worlds.isEmpty()) {
                if (worlds.contains(player.getLocation().getWorld().getName())) {
                    fc.complete(new ItemStack(Material.AIR));
                    return;
                }
            }

            fc.complete(builder.getItem());
        });

        fc.thenAccept(item -> {
            if(item == null)
                return;

            int slot = manager.getInt(StorageKey.MENU_ITEM_SLOT);
            player.getInventory().setItem(slot, item);
            player.updateInventory();
        });
    }

}
