package nl.iobyte.themepark.listeners;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.attraction.AttractionService;
import nl.iobyte.themepark.api.attraction.objects.Attraction;
import nl.iobyte.themepark.api.config.ConfigurationManager;
import nl.iobyte.themepark.api.config.enums.StorageKey;
import nl.iobyte.themepark.api.message.MessageKey;
import nl.iobyte.themepark.api.message.Text;
import nl.iobyte.themepark.api.sign.SignManager;
import nl.iobyte.themepark.api.sign.objects.StatusSign;
import org.bukkit.Location;
import org.bukkit.block.Block;
import org.bukkit.block.Sign;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.block.SignChangeEvent;
import org.bukkit.event.player.PlayerInteractEvent;

public class StatusSignListener implements Listener {

    private final AttractionService service = ThemePark.getInstance().getAPI().getAttractionService();
    private final SignManager manager = ThemePark.getInstance().getAPI().getSignManager();
    private final ConfigurationManager config = ThemePark.getInstance().getAPI().getConfigurationManager();

    @EventHandler
    public void onPlace(SignChangeEvent e) {
        if(!e.getLine(0).equalsIgnoreCase(config.getString(StorageKey.SIGN_STATUS_NAME)))
            return;

        String id = e.getLine(1);
        if(id == null || id.isEmpty() || !service.hasAttraction(id))
            return;

        Attraction attraction = service.getAttraction(id);
        e.setLine(0, Text.color(config.getString(StorageKey.SIGN_STATUS_TITLE)));
        e.setLine(1, Text.color(attraction.getName()));
        e.setLine(2, Text.color(attraction.getStatus().getColor() + attraction.getStatus().getName()));

        Location location = e.getBlock().getLocation();
        StatusSign statusSign = new StatusSign(attraction, location);
        if(manager.addSign(statusSign))
            e.getPlayer().sendMessage(Text.color("&6&lThemePark &f➢ &aSuccessfully created a status sign"));
    }

    @EventHandler
    public void onBreak(BlockBreakEvent e) {
        if(e.getBlock() == null)
            return;

        Block block = e.getBlock();
        if(!(block.getState() instanceof Sign))
            return;

        Sign sign = (Sign) block.getState();
        if(!sign.getLine(0).equals(Text.color(config.getString(StorageKey.SIGN_STATUS_TITLE))))
            return;

        Attraction attraction = service.getAttractionByName(sign.getLine(1));
        Location location = e.getBlock().getLocation();

        StatusSign s = new StatusSign(attraction, location);
        if(manager.removeSign(s))
            e.getPlayer().sendMessage(Text.color("&6&lThemePark &f➢ &aSuccessfully removed a status sign"));

    }

    @EventHandler
    public void onClick(PlayerInteractEvent e) {
        if(e.getAction() != Action.RIGHT_CLICK_BLOCK)
            return;

        Block b = e.getClickedBlock();
        if(!(b.getState() instanceof Sign))
            return;

        Sign sign = (Sign) b.getState();
        if(!sign.getLine(0).equals(Text.color(config.getString(StorageKey.SIGN_STATUS_TITLE))))
            return;

        Attraction attraction = service.getAttractionByName(sign.getLine(1));
        if(!manager.isSign(attraction, sign.getLocation()))
            return;

        Location location = attraction.getLocation();
        if(location == null)
            return;

        Player player = e.getPlayer();
        if(!attraction.getStatus().canTeleport()) {
            String message = MessageKey.ATTRACTION_TELEPORT_STATUS.getMessage();
            message = message.replace("{name}", attraction.getName());
            message = message.replace("{status}", attraction.getStatus().getColor() + attraction.getStatus().getName());
            player.sendMessage(Text.color(message));
            return;
        }

        player.teleport(location);
        String message = MessageKey.ATTRACTION_TELEPORT_SUCCESS.getMessage();
        message = message.replace("{name}", attraction.getName());
        player.sendMessage(Text.color(message));
    }

}
